/* sha1dc.c - SHA1DC hash function
 *	Copyright (C) 1998, 1999, 2000, 2001 Free Software Foundation, Inc.
 *      Copyright (C) 2021 Vitaly Chikunov <vt@altlinux.org>.
 *      Based on sha1.c
 *
 * Please see below for more legal information!
 *
 * This file is part of GnuPG.
 *
 * GnuPG is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 3 of the License, or
 * (at your option) any later version.
 *
 * GnuPG is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, see <http://www.gnu.org/licenses/>.
 */


#include <config.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <assert.h>
#include <sha1dc/sha1.h>
#include "util.h"
#include "memory.h"
#include "algorithms.h"
#include "bithelp.h"

typedef struct {
    SHA1_CTX ctx;
    byte buf[20];
} SHA1DC_CONTEXT;


void
sha1dc_init( SHA1DC_CONTEXT *hd )
{
    /* Security options are enabled by default. */
    SHA1DCInit(&hd->ctx);
}


/* Update the message digest with the contents
 * of INBUF with length INLEN.
 */
static void
sha1dc_write( SHA1DC_CONTEXT *hd, byte *inbuf, size_t inlen)
{

    SHA1DCUpdate(&hd->ctx, inbuf, inlen);
}


/* The routine final terminates the computation and
 * returns the digest.
 * Returns: 20 bytes representing the digest.
 */

static void
sha1dc_final(SHA1DC_CONTEXT *hd)
{
    SHA1DCFinal(hd->buf, &hd->ctx);
}

static byte *
sha1dc_read( SHA1DC_CONTEXT *hd )
{
    return hd->buf;
}


/****************
 * Shortcut functions which puts the hash value of the supplied buffer
 * into outbuf which must have a size of 20 bytes.
 */
void
sha1dc_hash_buffer (char *outbuf, const char *buffer, size_t length)
{
  SHA1DC_CONTEXT hd;

  sha1dc_init (&hd);
  sha1dc_write (&hd, (byte*)buffer, length);
  sha1dc_final (&hd);
  memcpy (outbuf, hd.buf, 20);
}


/****************
 * Return some information about the algorithm.  We need algo here to
 * distinguish different flavors of the algorithm.
 * Returns: A pointer to string describing the algorithm or NULL if
 *	    the ALGO is invalid.
 */
const char *
sha1dc_get_info( int algo, size_t *contextsize,
                 byte **r_asnoid, int *r_asnlen, int *r_mdlen,
                 void (**r_init)( void *c ),
                 void (**r_write)( void *c, byte *buf, size_t nbytes ),
                 void (**r_final)( void *c ),
                 byte *(**r_read)( void *c )
               )
{
    static byte asn[15] = /* Object ID is 1.3.14.3.2.26 */
		    { 0x30, 0x21, 0x30, 0x09, 0x06, 0x05, 0x2b, 0x0e, 0x03,
		      0x02, 0x1a, 0x05, 0x00, 0x04, 0x14 };

    *contextsize = sizeof(SHA1DC_CONTEXT);
    *r_asnoid = asn;
    *r_asnlen = DIM(asn);
    *r_mdlen = 20;
    *(void  (**)(SHA1DC_CONTEXT *))r_init                 = sha1dc_init;
    *(void  (**)(SHA1DC_CONTEXT *, byte*, size_t))r_write = sha1dc_write;
    *(void  (**)(SHA1DC_CONTEXT *))r_final                = sha1dc_final;
    *(byte *(**)(SHA1DC_CONTEXT *))r_read                 = sha1dc_read;

    return "SHA1DC";
}
